<?php
// $Id$
/**
 * @file
 * Defines the wowguild entity, and functions and class related to it.
 */

/**
 * Called from hook_entity_info().
 */
function wowtoon_entity_wowguild_info() {
  $return = array(
    'wowguild' => array(
      'label' => t('WoW Guild'),
      'controller class' => 'WowGuildController',
      'base table' => 'wowguild_entity',
      'fieldable' => TRUE,
      'entity keys' => array(
        'id' => 'gid',
      ),
      'bundles' => array(
        'wowguild' => array(
          'label' => t('WoWGuild'),
          'admin' => array(
            'path' => 'admin/config/wowguild/wowguild',
            'access arguments' => array('access content'),
          ),
        ),
      ),
      'label callback' => 'entity_class_label',
      'uri callback' => 'entity_class_uri',
      'view modes' => array(
        'full' => array(
          'label' => t('Guild'),
          'custom settings' => FALSE,
        ),
        'banner' => array(
          'label' => t('banner'),
          'custom settings' => FALSE,
        ),
        'popup' => array(
          'label' => t('popup'),
          'custom settings' => FALSE,
        ),
        'teaser' => array(
          'label' => t('teaser'),
          'custom settings' => FALSE,
        ),
      ),
    ),
  );
  return $return;  
}


/**
 * Main class for wowguild entities.
 */
class WowGuildController extends DrupalDefaultEntityController {
  /*
   * Make variables more accessable.
   */
  protected function attachLoad(&$guilds, $revision_id = FALSE) {
    parent::attachLoad($guilds, $revision_id);
    foreach ($guilds as $key => $value) {
      if (!empty($guilds[$key]->raid_progression)) {
        $guilds[$key]->raid_progression = unserialize($guilds[$key]->raid_progression);
      }
      if (!empty($guilds[$key]->tabard)) {
        $guilds[$key]->tabard = unserialize($guilds[$key]->tabard);
      }
      $guilds[$key]->nameatrealm = $guilds[$key]->name . '@' . $guilds[$key]->realm;
    }
  }
}

/**
 * Load multiple test entities based on certain conditions.
 *
 * @param $pids
 *   An array of entity IDs.
 * @param $conditions
 *   An array of conditions to match against the {entity} table.
 * @param $reset
 *   A boolean indicating that the internal cache should be reset.
 * @return
 *   An array of test entity objects, indexed by pid.
 */
function wowguild_load_multiple($gids = array(), $conditions = array(), $reset = FALSE) {
  return entity_load('wowguild', $gids, $conditions, $reset);
}

/**
 * hook for menu %wowtoon items
 * 
 * @param unknown_type $tid
 */
function wowguild_load($gid, $reset = FALSE) {
  if (!empty($gid)) {
    $guilds = wowguild_load_multiple(array($gid), array(), $reset);
    return $guilds?reset($guilds):FALSE;
  }
  else {
    return FALSE;
  }
}
/**
 * 
 * @param string $realm - Realm of guild to load.
 * @param string $name - Name of guild to load.
 * 
 * @return
 *   {wowguild} entity of guild or FALSE.
 */
function wowguild_load_by_realm_name($realm = NULL, $name = NULL) {
  $guild_key = wowtoon_getkey($realm, $name);
  $guilds = wowguild_load_multiple(array(), array('guild_key' => $guild_key));
  return reset($guilds);
}

/**
 * Load members of guild.
 */
function wowguild_load_members(&$guild) {
  $tids = db_query("SELECT tid FROM {wowtoon_entity} WHERE gid = :gid", array(':gid' => $guild->gid))->fetchCol();
  $guild->members = wowtoon_load_multiple($tids);
}

/**
 * Save {wowguild} entity.
 */
function wowguild_save(&$edit) {
  $edit->guild_key = wowtoon_getkey($edit->realm, $edit->name);
  field_attach_presave('wowguild', $edit);
  
  if (!empty($edit->gid)) {
    drupal_write_record('wowguild_entity', $edit, 'gid');
    field_attach_update('wowguild', $edit);
    module_invoke_all('entity_update', 'wowguild', $edit);
    return $edit;
  }
  drupal_write_record('wowguild_entity', $edit);
  field_attach_insert('wowguild', $edit);
  module_invoke_all('entity_insert', 'wowguild', $edit);
  return $edit;
}

/**
 * Build content for specified view_mode
 * 
 * @param wowguild $node
 * @param string $view_mode
 *   - full, teaser, popup, banner
 * @param string $langcode
 */
function wowguild_build_content(&$guild, $view_mode = 'full', $langcode = NULL) {
  if (!isset($langcode)) {
    $langcode = $GLOBALS['language_content']->language;
  }

  // Remove previously built content, if exists.
  $guild->content = array();

  // Build fields content.
  // In case of a multiple view, node_view_multiple() already ran the
  // 'prepare_view' step. An internal flag prevents the operation from running
  // twice.
  field_attach_prepare_view('wowguild', array($guild->gid => $guild), $view_mode);
  entity_prepare_view('wowguild', array($guild->gid => $guild));
  $guild->content += field_attach_view('wowguild', $guild, $view_mode, $langcode);

  // Allow modules to make their own additions to the node.
  module_invoke_all('wowguild_view', $guild, $view_mode, $langcode);
  module_invoke_all('entity_view', $guild, 'wowguild', $view_mode, $langcode);
}

function wowguild_page_title($guild) {
  if (isset($guild->name)) {
    return $guild->name . ' on ' . $guild->realm;
  }
}

function wowguild_lookup($realm, $name) {
  $guild = wowguild_load_by_realm_name($realm, $name);
  if ($guild) {
    drupal_goto('guild/' . $guild->gid);
  }
  else {
    return t('%name on %realm could not be found', array('%name' => $name, '%realm' => $realm));
  }
}

/**
 * View a specific guild.
 * 
 * @param {wowguild} $guild
 * @param string $view_mode
 * @param string $langcode
 */
function wowguild_view($guild, $view_mode = 'full', $langcode = NULL) {
  if ($guild->updated == 0) {
    module_load_include('inc', 'wowtoon', 'wowtoon.update');
    $guild = wowguild_update_armory($guild->gid);
    wowguild_update_feed($guild);
    wowguild_update_progression($guild);
    $guild = wowguild_load($guild->gid, TRUE);
  }
  if (!isset($langcode)) {
    $langcode = $GLOBALS['language_content']->language;
  }

  // Populate $node->content with a render() array.
  wowguild_build_content($guild, $view_mode, $langcode);

  $build = $guild->content;
  // We don't need duplicate rendering info in node->content.
  unset($guild->content);

  $build = array_merge($build, array(
    '#theme' => 'wowguild',
    '#guild' => $guild,
    '#view_mode' => $view_mode,
    '#language' => $langcode,
  ));
  if (module_exists('views')) {
    $build['members_view']['#weight'] = 20;
    if (module_exists('wowguild') && variable_get('wowguild_guild_gid', -999) == $guild->gid) {
      $build['members_view']['#markup'] = views_embed_view('toon_views', 'guild_roster', $guild->gid);
    } 
    else {
      $build['members_view']['#markup'] = views_embed_view('toon_views', 'display_guild', $guild->gid);
    }
  }

  // Allow modules to modify the structured node.
  $type = 'wowguild';
  drupal_alter(array('wowguild_view', 'entity_view'), $build, $type);

  return $build;
}

/**
 * Return URL to guild tabard.  Create it if needed.
 * 
 */
function wowtoon_guild_tabard_img($guild) {
  // Verify GD is installed
  if (function_exists('imagelayereffect')) {
    $real_tabard_directory = variable_get('wowtoon_html5_tabards_cache_path', 'public://tabardcache');
    if (substr($real_tabard_directory, 0, 9) == 'public://') {
      $url_tabard_directory = file_create_url($real_tabard_directory);
      $real_tabard_directory = drupal_realpath($real_tabard_directory);
    }
    else {
      $url_tabard_directory = variable_get('wowtoon_html5_tabards_cache_url', file_create_url('public://tabardcache'));
    }
    
    $imageFile = sprintf('tabard_%d_%d_%s_%d_%s_%s.png',
      $guild->factionid,
      $guild->tabard['icon'],
      _wowtoon_verify_hex($guild->tabard['iconColor']),
      $guild->emblem->border,
      _wowtoon_verify_hex($guild->tabard['borderColor']),
      _wowtoon_verify_hex($guild->tabard['backgroundColor'])
    );
    $urlFile = $url_tabard_directory . '/' .  $imageFile;
    $imageFile = $real_tabard_directory . '/' . $imageFile;
    
    if (!file_exists($imageFile)) {
      $files = array(
        'emblem' => dirname(__FILE__) . sprintf("/images/emblems/emblem_%02d.png",$guild->tabard['icon']),
        'border' => dirname(__FILE__) . sprintf("/images/borders/border_%02d.png",$guild->tabard['border']),
        'ring' => dirname(__FILE__) . "/images/static/ring-" . ($guild->factionid == 0?'alliance':'horde') . ".png",
        'shadow' => dirname(__FILE__) . "/images/static/shadow_00.png",
        'bg' => dirname(__FILE__) . "/images/static/bg_00.png",
        'overlay' => dirname(__FILE__) . "/images/static/overlay_00.png",
        'hooks' => dirname(__FILE__) . "/images/static/hooks.png"
      );
      
      $imgOut = imagecreatetruecolor(215, 230);
      imagesavealpha($imgOut,true);
      imagealphablending($imgOut, true);
      $trans_colour = imagecolorallocatealpha($imgOut, 0, 0, 0, 127);
      imagefill($imgOut, 0, 0, $trans_colour);
      
      $ring = imagecreatefrompng($files['ring']);
      $ring_size = getimagesize($files['ring']);
      
      $emblem = imagecreatefrompng($files['emblem']);
      $emblem_size = getimagesize($files['emblem']);
      imagelayereffect($emblem, IMG_EFFECT_OVERLAY);
      $emblemcolor = preg_replace('/^ff/i','',$guild->tabard['iconColor']);
      $color_r = hexdec(substr($emblemcolor,0,2));
      $color_g = hexdec(substr($emblemcolor,2,2));
      $color_b = hexdec(substr($emblemcolor,4,2));
      imagefilledrectangle($emblem,0,0,$emblem_size[0],$emblem_size[1],imagecolorallocatealpha($emblem, $color_r, $color_g, $color_b,0));
      
      
      $border = imagecreatefrompng($files['border']);
      $border_size = getimagesize($files['border']);
      imagelayereffect($border, IMG_EFFECT_OVERLAY);
      $bordercolor = preg_replace('/^ff/i','',$guild->tabard['borderColor']);
      $color_r = hexdec(substr($bordercolor,0,2));
      $color_g = hexdec(substr($bordercolor,2,2));
      $color_b = hexdec(substr($bordercolor,4,2));
      imagefilledrectangle($border,0,0,$border_size[0]+100,$border_size[0]+100,imagecolorallocatealpha($border, $color_r, $color_g, $color_b,0));
      
      $shadow = imagecreatefrompng($files['shadow']);
      
      $bg = imagecreatefrompng($files['bg']);
      $bg_size = getimagesize($files['bg']);
      imagelayereffect($bg, IMG_EFFECT_OVERLAY);
      $bgcolor = preg_replace('/^ff/i','',$guild->tabard['backgroundColor']);
      $color_r = hexdec(substr($bgcolor,0,2));
      $color_g = hexdec(substr($bgcolor,2,2));
      $color_b = hexdec(substr($bgcolor,4,2));
      imagefilledrectangle($bg,0,0,$bg_size[0]+100,$bg_size[0]+100,imagecolorallocatealpha($bg, $color_r, $color_g, $color_b,0));
      
      $overlay = imagecreatefrompng($files['overlay']);
      $overlay_size = getimagesize($files['overlay']);
      $hooks = imagecreatefrompng($files['hooks']);
      $hooks_size = getimagesize($files['hooks']);
      
      $x = 20;
      $y = 23;
      
      $size = getimagesize($files['shadow']);
      
      imagecopy($imgOut,$ring,0,0,0,0, $ring_size[0],$ring_size[1]);
      imagecopy($imgOut,$shadow,$x,$y,0,0, $size[0],$size[1]);
      imagecopy($imgOut,$bg,$x,$y,0,0, $bg_size[0],$bg_size[1]);
      imagecopy($imgOut,$emblem,$x+17,$y+30,0,0, $emblem_size[0],$emblem_size[1]);
      imagecopy($imgOut,$border,$x+13,$y+15,0,0, $border_size[0],$border_size[1]);
      imagecopy($imgOut,$overlay,$x,$y+2,0,0, $overlay_size[0],$overlay_size[1]);
      imagecopy($imgOut,$hooks,$x-2,$y,0,0, $hooks_size[0],$hooks_size[1]);   
      
      // Verify directory exists
      if (!file_exists($real_tabard_directory)) {
        drupal_mkdir($real_tabard_directory, NULL, TRUE);
      }
      
      // Write file.
      imagepng($imgOut,$imageFile);
    
      imagedestroy($ring);
      imagedestroy($shadow);
      imagedestroy($bg);
      imagedestroy($overlay);
      imagedestroy($border);
      imagedestroy($emblem);
      imagedestroy($hooks);
      imagedestroy($imgOut);
    }
    return $urlFile;
  }
  else {
    // no GD compiled with PHP
    if ($guild->factionId == 0) {
      return 'http://us.battle.net/wow/static/images/guild/tabards/default-alliance.png';
    }
    else {
      return 'http://us.battle.net/wow/static/images/guild/tabards/default-horde.png';
    }
  }
  
}

/**
 * Remove any characters that are not valid hexidecimal characters (0-9,a-h)
 * 
 * @param unknown_type $str
 */
function _wowtoon_verify_hex($str) {
  if(preg_match('/^[a-f0-9]{8}$/i', $str)) {
    return $str;
  }
  else {
    return 0;
  }
}

function wowtoon_admin_wowguild() {
    $header = array(
    array('data' => t('Name'), 'field' => 'g.name', 'sort' => 'asc'),
    array('data' => t('Realm'), 'field' => 'g.realm'),
    // array('data' => t('Owner'), 'field' => 'u.username'),  // Guilds cannot be owned (yet)
    'Actions'
  );
  
  $query = db_select('wowguild_entity', 'g')->extend('PagerDefault')->extend('TableSort')->orderByHeader($header);
  $query->fields('g', array('gid', 'name', 'realm'));
  $query->leftJoin('users', 'u', 'u.uid = g.uid');
  $query->fields('u', array('name', 'uid'));
  /*
  $query->join('field_data_wowtoon_name', 'n', 'n.entity_id = g.gid AND n.entity_type = :entity_type AND n.deleted = 0', array(':entity_type' => 'wowguild'));
  $query->fields('n', array('wowtoon_name_value'));
  $query->join('field_data_wowtoon_realm', 'r', 'r.entity_id = g.gid AND r.entity_type = :entity_type AND r.deleted = 0', array(':entity_type' => 'wowguild'));
  $query->fields('r', array('wowtoon_realm_value'));
  */
  $query->limit(10);
  $results = $query->execute();
  
  
  $rows = array();
  foreach ($results as $guild) {
    $row = array();
    $row[] = l($guild->name, 'guild/' . $guild->gid);
    $row[] = $guild->realm;
    
    // Guilds cannot be owned (yet)
    //$row[] = l($guild->name, 'user/' . $guild->uid);
    
    $actions = array();
    $actions[] = l(t('view'), 'guild/' . $guild->gid);
    $row[] = implode(' ', $actions);
    $rows[] = $row;
  }
  
  $build['table'] = array(
    '#prefix' => '<h3>' . t('WoW Guilds') . '</h3>',
    '#theme' => 'table',
    '#header' => $header,
    '#rows' => $rows
  );
  $build['pager'] = array(
    '#theme' => 'pager',
  );
  return $build;
}